@@ -6,15 +6,13 @@ require_once('session_handler.php');
 
 session_start();
 
-if (empty($_SESSION['test'])) {
+if (!isset($_SESSION['test'])) {
     $_SESSION['test'] = 0;
 } else {
     $_SESSION['test']++;
 }
 
-header('Content-Type: text/plain');
-
 echo 'You reloaded that page ' . $_SESSION['test'] . ' time(s).' . "\n" .
-  'Your session link is: [' . SID . ']' . "\n";
+  'Your session id is: [' . session_id() . ']' . "\n";
 
 ?>
\ No newline at end of file
