@@ -17,7 +17,7 @@ void fd_nonblock(const int fd)
 
 int safe_write(const int fd, const void *buf_, size_t count)
 {
-    ssize_t written;    
+    ssize_t written;
     register const char *buf = buf_;
 
     while (count > (size_t) 0U) {
@@ -46,7 +46,8 @@ static void client_disconnect(Client * const client)
         (void) close(client->client_fd);
         client->client_fd = -1;
     }
-    if (client->read_mapped_zone != NULL) {
+    if (client->read_mapped_zone != NULL &&
+        client->read_mapped_zone != MAP_FAILED) {
         (void) munmap(client->read_mapped_zone,
                       client->read_mapped_zone_length);
         client->read_mapped_zone = NULL;
@@ -92,11 +93,11 @@ static void returncode_bufferev_error_cb(struct bufferevent * const bufferev,
                                          short what, void *client_)
 {
     Client * const client = client_;
-    
+
     (void) bufferev;
     logfile(LOG_DEBUG, _("Error %d when sending return code to fd #%d"),
             (int) what, client->client_fd);
-    client_disconnect(client);    
+    client_disconnect(client);
 }
 
 static int validate_key(const unsigned char *buf, size_t size)
@@ -111,7 +112,7 @@ static int validate_key(const unsigned char *buf, size_t size)
             return -1;
         }
     } while (size > (size_t) 0U);
-    
+
     return 0;
 }
 
@@ -144,7 +145,7 @@ static int client_process_store(Client * const client)
     memcpy(store_file_pnt, client->read_buf + (size_t) 9U, client->key_len);
     store_file_pnt += client->key_len;
     *store_file_pnt = 0;
-    
+
     sizeof_store_file_tmp = (sizeof "/") - (size_t) 1U +
         sizeof STOREFILE_TMP_PREFIX - (size_t) 1U +
         client->key_len + (size_t) 1U;
@@ -161,8 +162,8 @@ static int client_process_store(Client * const client)
     memcpy(store_file_tmp_pnt,
            client->read_buf + (size_t) 9U, client->key_len);
     store_file_tmp_pnt += client->key_len;
-    *store_file_tmp_pnt = 0;    
-    
+    *store_file_tmp_pnt = 0;
+
     logfile(LOG_DEBUG, _("Creating [%s]"), store_file_tmp);
     if ((client->write_fd =
          open(store_file_tmp, O_CREAT | O_NOFOLLOW | O_WRONLY | O_TRUNC,
@@ -193,14 +194,15 @@ static int client_process_store(Client * const client)
     }
     ALLOCA_FREE(store_file);
     ALLOCA_FREE(store_file_tmp);
-    if ((client->returncode_bufev = 
-         bufferevent_new(client->client_fd, 
+    if ((client->returncode_bufev =
+         bufferevent_new(client->client_fd,
                          returncode_bufferev_read_cb,
                          returncode_bufferev_write_cb,
                          returncode_bufferev_error_cb,
                          client)) == NULL) {
         logfile(LOG_WARNING, _("Unable to create a bufferevent for fd #%d"),
                 client->client_fd);
+        return -1;
     }
     (void) bufferevent_write(client->returncode_bufev, (void *) RETURNCODE_OK,
                              sizeof RETURNCODE_OK - (size_t) 1U);
@@ -218,7 +220,7 @@ static int client_process_fetch(Client * const client)
     logfile(LOG_DEBUG, _("client_process_fetch, fd #%d"), client->client_fd);
     if (client->offset_read_buf < (size_t) 6U) {
         return -1;
-    }    
+    }
     sizeof_store_file = (sizeof "/") - (size_t) 1U +
         client->key_len + (size_t) 1U;
     if ((store_file = ALLOCA(sizeof_store_file)) == NULL) {
@@ -228,7 +230,7 @@ static int client_process_fetch(Client * const client)
     store_file_pnt = store_file;
     *store_file_pnt++ = '/';
     if (validate_key(client->read_buf + (size_t) 5U, client->key_len) != 0) {
-        ALLOCA_FREE(store_file);        
+        ALLOCA_FREE(store_file);
         logfile(LOG_WARNING, _("Invalid key name"));
         return -1;
     }
@@ -237,45 +239,47 @@ static int client_process_fetch(Client * const client)
     *store_file_pnt = 0;
     logfile(LOG_DEBUG, _("Reading [%s]"), store_file);
     if ((client->read_fd = open(store_file, O_RDONLY)) == -1) {
-        logfile(LOG_WARNING, _("Unable to fetch [%s]: [%s]"), store_file,
+        logfile(LOG_DEBUG, _("Unable to fetch [%s]: [%s]"), store_file,
                 strerror(errno));
         ALLOCA_FREE(store_file);
         return -1;
     }
     logfile(LOG_DEBUG, _("[%s] = fd #%d"), store_file, client->read_fd);
-    ALLOCA_FREE(store_file);
-    if (fstat(client->read_fd, &st) != 0) {
+    if (fstat(client->read_fd, &st) != 0 || st.st_size <= (off_t) 0) {        
         logfile(LOG_DEBUG, _("Unable to stat a previous key (%s) : [%s]"),
                 store_file, strerror(errno));
         (void) close(client->read_fd);
         client->read_fd = -1;
         ALLOCA_FREE(store_file);
-        return -1;        
+        return -1;
     }
     if ((client->read_mapped_zone = mmap(NULL, st.st_size, PROT_READ,
                                          MAP_SHARED, client->read_fd,
-                                         (off_t) 0)) == NULL) {
+                                         (off_t) 0)) == MAP_FAILED) {
         logfile(LOG_WARNING, _("Unable to map in memory: [%s]"),
                 strerror(errno));
+        client->read_mapped_zone = NULL;
         (void) close(client->read_fd);
         client->read_fd = -1;
         ALLOCA_FREE(store_file);
         return -1;
     }
+    ALLOCA_FREE(store_file);
     client->read_mapped_zone_length = st.st_size;
-    if ((client->returncode_bufev = 
-         bufferevent_new(client->client_fd, 
+    if ((client->returncode_bufev =
+         bufferevent_new(client->client_fd,
                          returncode_bufferev_read_cb,
                          returncode_bufferev_write_cb,
                          returncode_bufferev_error_cb,
                          client)) == NULL) {
         logfile(LOG_WARNING, _("Unable to create a bufferevent for fd #%d"),
                 client->client_fd);
+        return -1;
     }
     (void) bufferevent_write(client->returncode_bufev,
                              client->read_mapped_zone,
                              client->read_mapped_zone_length);
-    
+
     return 0;
 }
 
@@ -288,7 +292,7 @@ static int client_process_delete(Client * const client)
     logfile(LOG_DEBUG, _("client_process_delete, fd #%d"), client->client_fd);
     if (client->offset_read_buf < (size_t) 6U) {
         return -1;
-    }    
+    }
     sizeof_store_file = (sizeof "/") - (size_t) 1U +
         client->key_len + (size_t) 1U;
     if ((store_file = ALLOCA(sizeof_store_file)) == NULL) {
@@ -298,7 +302,7 @@ static int client_process_delete(Client * const client)
     store_file_pnt = store_file;
     *store_file_pnt++ = '/';
     if (validate_key(client->read_buf + (size_t) 5U, client->key_len) != 0) {
-        ALLOCA_FREE(store_file);        
+        ALLOCA_FREE(store_file);
         logfile(LOG_WARNING, _("Invalid key name"));
         return -1;
     }
@@ -312,18 +316,19 @@ static int client_process_delete(Client * const client)
         ALLOCA_FREE(store_file);
         return -1;
     }
-    if ((client->returncode_bufev = 
-         bufferevent_new(client->client_fd, 
+    if ((client->returncode_bufev =
+         bufferevent_new(client->client_fd,
                          returncode_bufferev_read_cb,
                          returncode_bufferev_write_cb,
                          returncode_bufferev_error_cb,
                          client)) == NULL) {
         logfile(LOG_WARNING, _("Unable to create a bufferevent for fd #%d"),
                 client->client_fd);
+        return -1;
     }
     (void) bufferevent_write(client->returncode_bufev, (void *) RETURNCODE_OK,
                              sizeof RETURNCODE_OK - (size_t) 1U);
-    
+
     return 0;
 }
 
@@ -336,12 +341,12 @@ static void client_read(const int client_fd, short event, void *client_)
     if (event == EV_TIMEOUT) {
         logfile(LOG_DEBUG, _("Timeout on descriptor #%d"), client->client_fd);
         client_disconnect(client);
-        return;                
+        return;
     }
     needed_size = client->offset_read_buf + read_chunk_size;
     if (client->sizeof_read_buf < needed_size) {
         unsigned char *tmp_buf;
-        
+
         if ((tmp_buf = realloc(client->read_buf,
                                needed_size)) == NULL) {
             logfile(LOG_ERR, _("Out of memory for client read buffers"));
@@ -356,7 +361,7 @@ static void client_read(const int client_fd, short event, void *client_)
          client->read_buf + client->offset_read_buf,
          client->sizeof_read_buf - client->offset_read_buf);
     if (readen <= (ssize_t) 0) {
-        if (errno == EINTR || errno == EAGAIN) {            
+        if (errno == EINTR || errno == EAGAIN) {
             logfile(LOG_DEBUG, _("Interrupted read for fd #%d: [%s]"),
                     client_fd, strerror(errno));
             event_add(&client->ev_client_read, &timeout);
@@ -368,52 +373,53 @@ static void client_read(const int client_fd, short event, void *client_)
         } else {
             logfile(LOG_DEBUG,
                     _("Descriptor #%d has disconnected with error [%s]"),
-                    strerror(errno));
+                    client->client_fd, strerror(errno));
         }
         client_disconnect(client);
-        return;        
+        return;
     }
     client->offset_read_buf += readen;
     logfile(LOG_DEBUG, _("Activity on client #%d (%d) offset=%lu"),
             client->client_fd, event,
-            (unsigned int) client->offset_read_buf);    
+            (unsigned long) client->offset_read_buf);
     if (client->client_command == CC_UNDEF) {
         if (client->read_buf[0] == CC_FETCH &&
             client->offset_read_buf > 5) {
             client->key_len =
-                client->read_buf[1] << 24 | client->read_buf[2] << 16 |
-                client->read_buf[3] << 8 | client->read_buf[4];
+                (size_t) client->read_buf[1] << 24 | (size_t) client->read_buf[2] << 16 |
+                (size_t) client->read_buf[3] << 8 | (size_t) client->read_buf[4];
             client->total_len = (size_t) 1U + (size_t) 4U + client->key_len;
             client->client_command = CC_FETCH;
         } else if (client->read_buf[0] == CC_DELETE &&
             client->offset_read_buf > 5) {
             client->key_len =
-                client->read_buf[1] << 24 | client->read_buf[2] << 16 |
-                client->read_buf[3] << 8 | client->read_buf[4];
+                (size_t) client->read_buf[1] << 24 | (size_t) client->read_buf[2] << 16 |
+                (size_t) client->read_buf[3] << 8 | (size_t) client->read_buf[4];
             client->total_len = (size_t) 1U + (size_t) 4U + client->key_len;
             client->client_command = CC_DELETE;
         } else if (client->read_buf[0] == CC_STORE &&
             client->offset_read_buf > 9) {
             client->key_len =
-                client->read_buf[1] << 24 | client->read_buf[2] << 16 |
-                client->read_buf[3] << 8 | client->read_buf[4];
+                (size_t) client->read_buf[1] << 24 | (size_t) client->read_buf[2] << 16 |
+                (size_t) client->read_buf[3] << 8 | (size_t) client->read_buf[4];
             client->data_len =
-                client->read_buf[5] << 24 | client->read_buf[6] << 16 |
-                client->read_buf[7] << 8 | client->read_buf[8];
+                (size_t) client->read_buf[5] << 24 | (size_t) client->read_buf[6] << 16 |
+                (size_t) client->read_buf[7] << 8 | (size_t) client->read_buf[8];
             client->total_len = (size_t) 1U + (size_t) 4U + (size_t) 4U +
                 client->key_len + client->data_len;
             client->client_command = CC_STORE;
         } else if (client->offset_read_buf > 0 &&
                    client->read_buf[0] != CC_STORE &&
-                   client->read_buf[0] != CC_FETCH) {
+                   client->read_buf[0] != CC_FETCH &&
+                   client->read_buf[0] != CC_DELETE) {
             logfile(LOG_WARNING, _("Unknown command [%d], good bye."),
                     (int) client->read_buf[0]);
             client_disconnect(client);
-            return;            
+            return;
         }
     }
     if (client->total_len > (size_t) 0U &&
-        client->offset_read_buf == client->total_len) {        
+        client->offset_read_buf == client->total_len) {
         if (client->client_command == CC_STORE) {
             if (client_process_store(client) < 0) {
                 client_disconnect(client);
@@ -456,7 +462,7 @@ static void client_read(const int client_fd, short event, void *client_)
 }
 
 void new_client(const int listen_fd, short event, void *ev)
-{    
+{
     Client *client;
     struct sockaddr_storage client_sa;
     socklen_t client_sa_len = sizeof client_sa;
@@ -490,7 +496,7 @@ void new_client(const int listen_fd, short event, void *ev)
     client->offset_read_buf = (size_t) 0U;
     client->client_command = CC_UNDEF;
     client->key_len = (size_t) 0U;
-    client->data_len = (size_t) 0U;    
+    client->data_len = (size_t) 0U;
     client->total_len = (size_t) 0U;
     client->write_fd = -1;
     client->read_fd = -1;
@@ -500,5 +506,5 @@ void new_client(const int listen_fd, short event, void *ev)
     event_set(&client->ev_client_read, client_fd, EV_READ,
               client_read, client);
     event_add(&client->ev_client_read, &timeout);
-    logfile(LOG_DEBUG, _("New client on fd #%d"), client_fd);    
+    logfile(LOG_DEBUG, _("New client on fd #%d"), client_fd);
 }
