@@ -88,7 +88,16 @@ static void clearargs(int argc, char **argv)
         while (env_nb > 0U) {
             env_nb--;
             /* Can any bad thing happen if strdup() ever fails? */
-            new_environ[env_nb] = strdup(environ[env_nb]);
+            if ((new_environ[env_nb] = strdup(environ[env_nb])) == NULL) {
+                unsigned int temp_env_nb = env_nb;
+                unsigned int orig_env_nb = env_nb;
+                while (temp_env_nb < (1U + orig_env_nb)) {
+                    free(new_environ[temp_env_nb]);
+                    temp_env_nb++;
+                }
+                free(new_environ);
+                abort();
+            }
         }
         environ = new_environ;
     }
@@ -99,7 +108,7 @@ static void clearargs(int argc, char **argv)
 #endif
 }
 
-static void setprogname(const char * const title)
+static void set_progname(const char * const title)
 {
 #ifndef NO_PROCNAME_CHANGE
 # ifdef HAVE_SETPROCTITLE
@@ -314,6 +323,7 @@ int main(int argc, char *argv[])
     }
     freeaddrinfo(res);
 
+    tzset();
     dodaemonize();
     if (chroot(storage_dir) != 0 || chdir("/") != 0) {
         logfile(LOG_ERR, _("Unable to chroot to [%s]: [%s]"), storage_dir,
@@ -331,11 +341,11 @@ int main(int argc, char *argv[])
     }
     if (expire_pid == (pid_t) 0) {
         (void) close(listen_fd);
-        setprogname("sharedanced [CLEANUP]");
+        set_progname("sharedanced [CLEANUP]");
         expire();
         _exit(0);
     }
-    setprogname("sharedanced [SERVER]");
+    set_progname("sharedanced [SERVER]");
     event_init();
     signal_set(&sigchld_ev, SIGCHLD, sigchld, &ev);
     signal_add(&sigchld_ev, NULL);
